/* global toString */

import { Promise } from "./promise";

function promiseAtLeast(expected_count, promises) {
  if (Object.prototype.toString.call(promises) !== "[object Array]") {
    throw new TypeError('You must pass an array to all.');
  }

  function canceller() {
    var promise;
    for (var i = 0; i < promises.length; i++) {
      promise = promises[i];

      if (promise && typeof promise.then === 'function' &&
          typeof promise.cancel === 'function') {
        promise.cancel();
      }
    }
  }

  return new Promise(function(resolve, reject, notify) {
    var results = [], remaining = promises.length,
    promise, remaining_count = promises.length - expected_count;

    if (remaining === 0) {
      if (expected_count === 1) {
        resolve();
      } else {
        resolve([]);
      }
    }

    function resolver(index) {
      return function(value) {
        resolveAll(index, value);
      };
    }

    function resolveAll(index, value) {
      results[index] = value;
      if (--remaining === remaining_count) {
        if (remaining_count === 0) {
          resolve(results);
        } else {
          resolve(value);
          canceller();
        }
      }
    }

    function notifier(index) {
      return function(value) {
        notify({"index": index, "value": value});
      };
    }

    function cancelAll(rejectionValue) {
      reject(rejectionValue);
      canceller();
    }

    for (var i = 0; i < promises.length; i++) {
      promise = promises[i];

      if (promise && typeof promise.then === 'function') {
        promise.then(resolver(i), cancelAll, notifier(i));
      } else {
        resolveAll(i, promise);
      }
    }
  }, canceller
  );
}

function all(promises) {
  return promiseAtLeast(promises.length, promises);
}

function any(promises) {
  return promiseAtLeast(1, promises);
}

export { all, any };
